@@ -5,7 +5,7 @@ from abc import ABC, abstractmethod
 from dataclasses import dataclass, field, fields
 from importlib.util import find_spec
 from logging import INFO
-from typing import TYPE_CHECKING, Any, Callable, Literal, cast
+from typing import TYPE_CHECKING, Any, Callable, Literal, Union, cast
 
 from litestar.exceptions import ImproperlyConfiguredException, MissingDependencyException
 from litestar.serialization.msgspec_hooks import _msgspec_json_encoder
@@ -167,6 +167,8 @@ class BaseLoggingConfig(ABC):
     """
     exception_logging_handler: ExceptionLoggingHandler | None
     """Handler function for logging exceptions."""
+    disable_stack_trace: set[Union[int, type[Exception]]]  # noqa: UP007
+    """Set of http status codes and exceptions to disable stack trace logging for."""
 
     @abstractmethod
     def configure(self) -> GetLogger:
@@ -244,6 +246,8 @@ class LoggingConfig(BaseLoggingConfig):
     """Should the root logger be configured, defaults to True for ease of configuration."""
     log_exceptions: Literal["always", "debug", "never"] = field(default="debug")
     """Should exceptions be logged, defaults to log exceptions when 'app.debug == True'"""
+    disable_stack_trace: set[Union[int, type[Exception]]] = field(default_factory=set)  # noqa: UP007
+    """Set of http status codes and exceptions to disable stack trace logging for."""
     traceback_line_limit: int = field(default=-1)
     """Max number of lines to print for exception traceback.
 
@@ -285,6 +289,7 @@ class LoggingConfig(BaseLoggingConfig):
             "log_exceptions",
             "propagate",
             "traceback_line_limit",
+            "disable_stack_trace",
         }
 
         if not self.configure_root_logger:
@@ -476,6 +481,8 @@ class StructLoggingConfig(BaseLoggingConfig):
     """Whether to cache the logger configuration and reuse."""
     log_exceptions: Literal["always", "debug", "never"] = field(default="debug")
     """Should exceptions be logged, defaults to log exceptions when 'app.debug == True'"""
+    disable_stack_trace: set[Union[int, type[Exception]]] = field(default_factory=set)  # noqa: UP007
+    """Set of http status codes and exceptions to disable stack trace logging for."""
     traceback_line_limit: int = field(default=-1)
     """Max number of lines to print for exception traceback.
 
@@ -536,6 +543,7 @@ class StructLoggingConfig(BaseLoggingConfig):
                     "traceback_line_limit",
                     "exception_logging_handler",
                     "pretty_print_tty",
+                    "disable_stack_trace",
                 )
             }
         )
